"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const services_1 = require("../services");
const getFiles_1 = require("../utils/getFiles");
class Create extends core_1.Command {
    async run() {
        const { args, flags } = await this.parse(Create);
        const { filename, needTTS, upload, onlyUpload } = flags;
        switch (args.option) {
            case 'tts':
                await tts({ filename });
                break;
            case 'youtube':
                await youtube({ filename, needTTS, upload, onlyUpload });
                break;
            case 'instagram':
                await instagram({ filename, needTTS, upload, onlyUpload });
                break;
            case 'podcast':
                await podcast({ filename, needTTS });
                break;
        }
    }
}
exports.default = Create;
Create.description = 'Create video and upload to destination';
Create.examples = ['<%= config.bin %> <%= command.id %> youtube -u -t'];
Create.flags = {
    filename: core_1.Flags.string({
        char: 'f',
        description: 'filename with content',
    }),
    needTTS: core_1.Flags.boolean({
        char: 't',
        description: "need to create TTS. If you haven't created TTS separately (with option tts), you can set this flag to create along the creation of video",
    }),
    upload: core_1.Flags.boolean({
        char: 'u',
        description: 'should upload result to destination (only works to YouTube and Instagram)',
    }),
    onlyUpload: core_1.Flags.boolean({
        description: 'Only upload result to destination (only works to YouTube and Instagram). Your video should be created separately, placed on tmp folder and be the last file created on it.',
    }),
};
Create.args = [
    {
        name: 'option',
        required: true,
        description: 'Format to create content',
        options: ['youtube', 'instagram', 'tts', 'podcast'],
    },
];
const tts = async ({ filename }) => {
    const content = await new services_1.GetContentService().execute(filename);
    await new services_1.TextToSpeechService(content).execute({
        synthesizeIntro: true,
        synthesizeEnd: true,
    });
};
const youtube = async ({ filename, needTTS, onlyUpload, upload, }) => {
    const content = await new services_1.GetContentService().execute(filename);
    if (!onlyUpload) {
        if (needTTS) {
            await new services_1.TextToSpeechService(content).execute({
                synthesizeIntro: true,
                synthesizeEnd: true,
            });
        }
        await new services_1.RetrieveAudioDuration(content).execute(needTTS || false, {
            haveIntro: true,
            haveEnd: true,
        });
        await new services_1.ExportDataService(content).execute('landscape');
        const bundle = await new services_1.BundleVideoService().execute();
        await new services_1.RenderVideoService(content).execute(bundle, 'landscape', true, 'youtube');
        await new services_1.CreateThumnailService(content).execute(bundle);
    }
    if (upload || onlyUpload) {
        const videoPath = await getFiles_1.getLatestFileCreated('mp4');
        const thumbnailPath = await getFiles_1.getLatestFileCreated('jpeg');
        await new services_1.YoutubeUploadService(content).execute(videoPath, thumbnailPath);
    }
};
const instagram = async ({ filename, needTTS, onlyUpload, upload, }) => {
    const content = await new services_1.GetContentService().execute(filename);
    if (!onlyUpload) {
        if (needTTS) {
            await new services_1.TextToSpeechService(content).execute({
                synthesizeIntro: false,
                synthesizeEnd: false,
            });
        }
        await new services_1.RetrieveAudioDuration(content).execute(needTTS || false, {
            haveIntro: false,
            haveEnd: false,
        });
        await new services_1.ExportDataService(content).execute('portrait');
        const bundle = await new services_1.BundleVideoService().execute();
        await new services_1.RenderVideoService(content).execute(bundle, 'portrait', false, 'instagram');
        await new services_1.CreateThumnailService(content).execute(bundle);
    }
    if (upload || onlyUpload) {
        const videoPath = await getFiles_1.getLatestFileCreated('mp4');
        const thumbnailPath = await getFiles_1.getLatestFileCreated('jpeg');
        await new services_1.InstagramUploadService(content).execute(videoPath, thumbnailPath);
    }
};
const podcast = async ({ filename, needTTS }) => {
    const content = await new services_1.GetContentService().execute(filename);
    if (needTTS) {
        await new services_1.TextToSpeechService(content).execute({
            synthesizeIntro: false,
            synthesizeEnd: false,
        });
    }
    await new services_1.RetrieveAudioDuration(content).execute(needTTS || false, {
        haveIntro: false,
        haveEnd: false,
    });
    await new services_1.ExportDataService(content).execute('square');
    const bundle = await new services_1.BundleVideoService().execute();
    await new services_1.CreateThumnailService(content).execute(bundle);
    await new services_1.CreatePodcastAudioFile(content).execute();
};
