"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const googleapis_1 = require("googleapis");
const node_html_parser_1 = require("node-html-parser");
const log_1 = require("../utils/log");
const _1 = require(".");
class MailToJsonService {
    constructor() {
        this.senderMail = 'newsletter@filipedeschamps.com.br';
        this.redirectUrl = 'http://localhost:3000/oauth2callback';
        if (!process.env.GOOGLE_CLIENT_ID) {
            log_1.error('Google Client ID is not defined', 'MailToJsonService');
            process.exit(1);
        }
        if (!process.env.GOOGLE_CLIENT_SECRET) {
            log_1.error('Google Client Secret is not defined', 'MailToJsonService');
            process.exit(1);
        }
        if (!process.env.GOOGLE_REFRESH_TOKEN) {
            log_1.error('Google Refresh Token is not defined', 'MailToJsonService');
            process.exit(1);
        }
        this.clientId = process.env.GOOGLE_CLIENT_ID;
        this.clientSecret = process.env.GOOGLE_CLIENT_SECRET;
        this.refreshToken = process.env.GOOGLE_REFRESH_TOKEN;
    }
    async execute() {
        try {
            log_1.log('Executing MailToJsonService', 'MailToJsonService');
            const auth = await this.getAccessToken();
            const { news, title } = await this.getNewsFromEmail(auth);
            await this.createContentFile(news, title);
        }
        catch (err) {
            log_1.error(`Failed at creating JSON file from mail \n${JSON.stringify(err)}`, 'MailToJsonService');
        }
    }
    async getAccessToken() {
        log_1.log('Getting access token', 'MailToJsonService');
        return new Promise(resolve => {
            const { OAuth2 } = googleapis_1.google.auth;
            const oauth2client = new OAuth2(this.clientId, this.clientSecret, this.redirectUrl);
            oauth2client.setCredentials({
                refresh_token: this.refreshToken,
            });
            oauth2client.refreshAccessToken(async (err, token) => {
                if (err || !token) {
                    log_1.error(`Failed at refreshing Google token \n${JSON.stringify(err)}`, 'MailToJsonService');
                    return;
                }
                oauth2client.setCredentials(token);
                resolve(oauth2client);
            });
        });
    }
    async getNewsFromEmail(auth) {
        var _a, _b, _c, _d, _e;
        log_1.log('Getting mail content', 'MailToJsonService');
        const gmail = googleapis_1.google.gmail({ version: 'v1', auth });
        const today = new Date();
        const yesterday = new Date(today);
        const tomorrow = new Date(today);
        tomorrow.setDate(tomorrow.getDate() + 1);
        yesterday.setDate(yesterday.getDate() - 1);
        const yesterdayFormatted = `${yesterday.getFullYear()}/${yesterday.getMonth() + 1}/${yesterday.getDate()}`;
        const tomorrowFormatted = `${tomorrow.getFullYear()}/${tomorrow.getMonth() + 1}/${tomorrow.getDate()}`;
        const mailList = await gmail.users.messages.list({
            userId: 'me',
            q: `from:(${this.senderMail}) after:${yesterdayFormatted} before:${tomorrowFormatted}`,
            maxResults: 1,
        });
        if (!((_a = mailList.data.messages) === null || _a === void 0 ? void 0 : _a.length) || !mailList.data.messages[0].id) {
            log_1.error('No mail found', 'MailToJsonService');
            return {};
        }
        const messageId = mailList.data.messages[0].id;
        const mail = await gmail.users.messages.get({
            userId: 'me',
            id: messageId,
        });
        if (!mail.data.payload || !((_b = mail.data.payload.parts) === null || _b === void 0 ? void 0 : _b.length)) {
            log_1.error('No mail payload found', 'MailToJsonService');
            return {};
        }
        const title = ((_c = mail.data.payload.headers) === null || _c === void 0 ? void 0 : _c.filter(header => header.name === 'Subject')[0].value) || '';
        const htmlBase64Mail = (_d = mail.data.payload) === null || _d === void 0 ? void 0 : _d.parts.filter(part => part.mimeType === 'text/html')[0];
        if (!htmlBase64Mail || !((_e = htmlBase64Mail.body) === null || _e === void 0 ? void 0 : _e.data)) {
            log_1.error('No mail body found', 'MailToJsonService');
            return {};
        }
        const htmlMail = Buffer.from(htmlBase64Mail.body.data, 'base64').toString();
        const parsed = node_html_parser_1.parse(htmlMail, {
            comment: false,
            blockTextElements: {
                script: false,
                noscript: false,
                style: false,
                pre: false,
            },
        });
        const paragraphs = parsed
            .querySelectorAll('tbody p')
            .map(p => p.innerHTML);
        const paragraphsWithoutIntroAndLinks = paragraphs
            .slice(1, paragraphs.length)
            .filter(paragraph => {
            return paragraph.indexOf('<a href') === -1;
        });
        const sanitizedText = paragraphsWithoutIntroAndLinks.map(paragraph => {
            const p = node_html_parser_1.parse(paragraph);
            return p.innerText
                .replace(/\r|\n|\t/g, '')
                .replace('      ', '')
                .replace(/"\w/g, '“')
                .replace(/\w"/g, '”');
        });
        return { news: sanitizedText, title };
    }
    async createContentFile(news, title) {
        const createContentTemplateService = new _1.CreateContentTemplateService();
        const today = new Date();
        const description = `${`${today.getDate()}`.padStart(2, '0') +
            `${today.getMonth() + 1}`.padStart(2, '0')}${today.getFullYear()}`;
        await createContentTemplateService.execute(description, {
            news,
            title,
        });
    }
}
exports.default = MailToJsonService;
