"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const renderer_1 = require("@remotion/renderer");
const log_1 = require("../utils/log");
const defaultPaths_1 = require("../config/defaultPaths");
const format_1 = tslib_1.__importDefault(require("../config/format"));
const bar_1 = tslib_1.__importDefault(require("../utils/CliProgress/bar"));
class RenderVideoService {
    constructor(content) {
        this.compositionId = 'Main';
        this.content = content;
    }
    async execute(bundle, videoFormat, withIntro, destination) {
        log_1.log(`Getting compositions from ${bundle}`, 'RenderVideoService');
        const tmpPath = await defaultPaths_1.getPath('tmp');
        const compositions = await renderer_1.getCompositions(bundle, {
            inputProps: {
                filename: `${this.content.timestamp}.json`,
                tmpPath,
            },
        });
        const video = compositions.find(c => c.id === this.compositionId);
        if (!video) {
            log_1.error(`Video not found`, 'RenderVideoService');
            return '';
        }
        const framesDir = await fs_1.default.promises.mkdtemp(path_1.default.join(tmpPath, 'frames-'));
        const outputVideoPath = path_1.default.resolve(tmpPath, `${this.content.timestamp}.mp4`);
        log_1.log(`Rendering frames`, 'RenderVideoService');
        let renderProgressBar = {};
        const { assetsInfo, frameCount } = await renderer_1.renderFrames({
            config: video,
            webpackBundle: bundle,
            onStart: ({ frameCount: total }) => {
                renderProgressBar = new bar_1.default({
                    initValue: 0,
                    total,
                    text: '[RenderVideoService] Progress {bar} {percentage}% | ETA: {eta}s | {value}/{total} | Rate: {rate}',
                });
            },
            onFrameUpdate: frame => {
                renderProgressBar.update(frame);
            },
            parallelism: null,
            outputDir: framesDir,
            inputProps: {
                filename: `${this.content.timestamp}.json`,
                withoutIntro: !withIntro,
                destination,
                tmpPath,
            },
            compositionId: this.compositionId,
            imageFormat: 'jpeg',
        });
        renderProgressBar.stop();
        log_1.log(`Stitching frames`, 'RenderVideoService');
        const stitchingProgressBar = new bar_1.default({
            initValue: 0,
            total: frameCount,
            text: '[RenderVideoService] Progress {bar} {percentage}% | ETA: {eta}s | {value}/{total} | Rate: {rate}',
        });
        await renderer_1.stitchFramesToVideo({
            dir: framesDir,
            fps: this.content.fps,
            width: format_1.default[videoFormat].width,
            height: format_1.default[videoFormat].height,
            outputLocation: outputVideoPath,
            force: true,
            imageFormat: 'jpeg',
            assetsInfo,
            onProgress: frame => {
                stitchingProgressBar.update(frame);
            },
        });
        fs_1.default.rmdirSync(framesDir, { recursive: true });
        stitchingProgressBar.stop();
        return outputVideoPath;
    }
}
exports.default = RenderVideoService;
