"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const googleapis_1 = require("googleapis");
const fs_1 = tslib_1.__importDefault(require("fs"));
const log_1 = require("../utils/log");
const bar_1 = tslib_1.__importDefault(require("../utils/CliProgress/bar"));
class YoutubeUploadService {
    constructor(content) {
        this.redirectUrl = 'http://localhost:3000/oauth2callback';
        this.descriptionCharactersLimit = 5000 - 100; // 100 is the safe area to avoid errors
        this.categoryIds = {
            Entertainment: 24,
            Education: 27,
            ScienceTechnology: 28,
        };
        this.defaultTags = [
            'CodeStack News',
            'Noticias',
            'Tecnologia',
            'Noticias rápidas',
            'tech news',
            'CodeStackNews',
            'technews',
            'ia',
            'programacao',
            'newsletter',
            'newsletterfilipedeschamps',
            'remotion',
            'dev',
            'desenvolvimento',
        ];
        this.content = content;
        if (!process.env.GOOGLE_CLIENT_ID) {
            log_1.error('Google Client ID is not defined', 'YoutubeUploadService');
            process.exit(1);
        }
        if (!process.env.GOOGLE_CLIENT_SECRET) {
            log_1.error('Google Client Secret is not defined', 'YoutubeUploadService');
            process.exit(1);
        }
        if (!process.env.YOUTUBE_REFRESH_TOKEN) {
            log_1.error('Youtube Refresh Token is not defined', 'YoutubeUploadService');
            process.exit(1);
        }
        this.clientId = process.env.GOOGLE_CLIENT_ID;
        this.clientSecret = process.env.GOOGLE_CLIENT_SECRET;
        this.refreshToken = process.env.YOUTUBE_REFRESH_TOKEN;
    }
    async execute(videoPath, thumbnailPath) {
        try {
            const auth = await this.getAccessToken();
            const videoId = await this.uploadVideo(auth, videoPath);
            await this.uploadThumbnail(auth, thumbnailPath, videoId);
        }
        catch (err) {
            log_1.error(`Failed at uploading video \n${JSON.stringify(err)}`, 'YoutubeUploadService');
        }
    }
    async getAccessToken() {
        return new Promise(resolve => {
            const { OAuth2 } = googleapis_1.google.auth;
            const oauth2client = new OAuth2(this.clientId, this.clientSecret, this.redirectUrl);
            oauth2client.credentials.refresh_token = this.refreshToken;
            oauth2client.refreshAccessToken((err, token) => {
                if (err || !token) {
                    log_1.error(`Failed at refreshing youtube token \n${JSON.stringify(err)}`, 'YoutubeUploadService');
                    return;
                }
                oauth2client.credentials = token;
                resolve(oauth2client);
            });
        });
    }
    getDescription() {
        var _a, _b;
        const descriptionArray = [];
        this.content.news.forEach(news => {
            var _a, _b;
            const [title, details] = news.text.split(': ');
            descriptionArray.push({
                title: details ? title.toUpperCase() : title,
                details: details
                    ? details.charAt(0).toUpperCase() + details.slice(1)
                    : '',
                url: (_b = (_a = news.shortLink) !== null && _a !== void 0 ? _a : news.url) !== null && _b !== void 0 ? _b : '',
            });
        });
        if (this.content.end) {
            const [title, details] = this.content.end.text.split(': ');
            descriptionArray.push({
                title: details ? title.toUpperCase() : title,
                details: details
                    ? details.charAt(0).toUpperCase() + details.slice(1)
                    : '',
                url: (_b = (_a = this.content.end.shortLink) !== null && _a !== void 0 ? _a : this.content.end.url) !== null && _b !== void 0 ? _b : '',
            });
        }
        const charactersLimitPerNews = this.descriptionCharactersLimit / descriptionArray.length;
        const newsDescriptionWithDetailsLengthLessThanLimitPerNewsArray = descriptionArray.filter(newsDescription => newsDescription.title.length +
            newsDescription.details.length +
            newsDescription.url.length <
            charactersLimitPerNews);
        const characterRemain = newsDescriptionWithDetailsLengthLessThanLimitPerNewsArray
            .map(newsDescription => charactersLimitPerNews -
            (newsDescription.title.length +
                newsDescription.details.length +
                newsDescription.url.length))
            .reduce((acc, detailsLength) => acc + detailsLength);
        const quantityOfNewsWithMoreThanLimitPerNews = descriptionArray.length -
            newsDescriptionWithDetailsLengthLessThanLimitPerNewsArray.length;
        const increaseEachDetailLimitWith = characterRemain / quantityOfNewsWithMoreThanLimitPerNews;
        const newCharactersLimitPerNews = charactersLimitPerNews + increaseEachDetailLimitWith;
        let description = '';
        descriptionArray.forEach(newsDescriptions => {
            description += `${newsDescriptions.title} \n`;
            if (newsDescriptions.details) {
                if (newsDescriptionWithDetailsLengthLessThanLimitPerNewsArray.includes(newsDescriptions)) {
                    description += `${newsDescriptions.details} \n`;
                }
                else {
                    const newsDetailsCharacterLimit = newCharactersLimitPerNews -
                        newsDescriptions.title.length -
                        newsDescriptions.url.length;
                    const trimmedDetails = newsDescriptions.details.substr(0, newsDetailsCharacterLimit);
                    description += `${trimmedDetails.substr(0, Math.min(trimmedDetails.length, trimmedDetails.lastIndexOf(' ')))}... \n`;
                }
            }
            if (newsDescriptions.url) {
                description += `${newsDescriptions.url} \n`;
            }
            description += '\n';
        });
        return description;
    }
    async uploadVideo(auth, videoPath) {
        const youtube = googleapis_1.google.youtube({ version: 'v3' });
        log_1.log('Uploading video to YouTube', 'YoutubeUploadService');
        const videoSize = Math.floor((fs_1.default.statSync(videoPath).size / 1000000) * 100) / 100;
        const uploadProgressBar = new bar_1.default({
            total: videoSize,
            initValue: 0,
            text: '[YoutubeUploadService] Progress {bar} {percentage}% | ETA: {eta}s | {value}/{total} Mb',
        });
        let title = `[CodeStack News] ${this.content.title}`;
        if (title.length >= 100) {
            const titleArray = title.split('/');
            titleArray.pop();
            title = titleArray.join('/');
        }
        const youtubeResponse = await youtube.videos.insert({
            auth,
            part: ['snippet', 'status'],
            requestBody: {
                snippet: {
                    title,
                    description: this.getDescription(),
                    tags: this.defaultTags,
                    categoryId: `${this.categoryIds.ScienceTechnology}`,
                },
                status: {
                    privacyStatus: 'public',
                    madeForKids: false,
                },
            },
            media: {
                body: fs_1.default.createReadStream(videoPath),
            },
        }, {
            onUploadProgress: event => uploadProgressBar.update(Math.floor(event.bytesRead / 10000) / 100),
        });
        uploadProgressBar.stop();
        if (!youtubeResponse.data.id) {
            throw new Error('Youtube video Id is not defined');
        }
        log_1.log(`Upload completed. Video is available at https://youtu.be/${youtubeResponse.data.id}`, 'YoutubeUploadService');
        return youtubeResponse.data.id;
    }
    async uploadThumbnail(auth, thumnailPath, videoId) {
        const youtube = googleapis_1.google.youtube({ version: 'v3' });
        log_1.log(`Uploading thumbnail`, 'YoutubeUploadService');
        await youtube.thumbnails.set({
            auth,
            videoId,
            media: {
                body: fs_1.default.createReadStream(thumnailPath),
            },
        });
    }
}
exports.default = YoutubeUploadService;
